#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# This script takes a multiple FFTS at different frequency ranges,
# and combines in both the saved data and the plot.
# FFT/Noise measurement script for suspensions electronics testing (SR785).  Modified from
# code by Yoichi Aso.
# cmreed 20100329
# DEA 15 Apr 2011
#
# DoubleFFTSR785.py [-f filename] [-i ip_address] [-a gpib_address] [{-b|--bandwidth} bandwidth]
#                 [{-n|--numpoints} num_of_points] [{-v|--averaging} num_of_avg] [--avgmode avg_mode] 
#                 [{-d|--dualchannel}] [--base baseFreq]
#                 [--ic1 ch1_input_coupling] [--ic2 ch2_input_coupling]
#                 [--ig1 ch1_input_ground] [--ig2 ch2_input_ground] [{-w|--window} window_func]
#                 [--title Title] [--memo Memo]                 
#                 

import re
import sys
import math
import optparse
import time
import netgpib
import SR785
import termstatus
#from pylab import *
#import matplotlib.pyplot as plt

#Parse options
usage = """usage: %prog [options]

This program remotely executes a spectrum measurement using SR785.
Various measurement conditions can be set using options.
The measurement result will be saved in FILENAME.txt
"""

filetime = time.strftime("%Y-%m-%d.%H%M%S",time.gmtime()) # Date string for data file (default)


# Options below set for SUS noise measurements
parser = optparse.OptionParser(usage=usage)
parser.add_option("-f", "--file", dest="filename",
                  help="Output file name without an extension", default=filetime)
parser.add_option("-i", "--ip",
                  dest="ipAddress", default="169.254.0.1",
                  help="IP address/Host name")
parser.add_option("-a", "--address",
                  dest="gpibAddress", type="int", default=10,
                  help="GPIB device address")
parser.add_option("-n", "--numpoints",
                  dest="numOfPoints", type="int", default="800",
                  help="Number of frequency points")
parser.add_option("-v", "--averaging",
                  dest="numAvg", type="int",default="10",
                  help="Number of averages")
parser.add_option("--avgmode",
                  dest="avgMode", type="string",default="RMS",
                  help="Averaging mode: None, Vector, RMS or PeakHold")
parser.add_option("-d", "--dualchannel",action="store_true",
                  dest="dual",
                  help="Set to the dual channel mode.")
parser.add_option("--ic1",
                  dest="inputCoupling1", type="string",default="DC",
                  help="CH1 input coupling. DC or AC")
parser.add_option("--ic2",
                  dest="inputCoupling2", type="string",default="DC",
                  help="CH2 input coupling. DC or AC")
parser.add_option("--ig1",
                  dest="inputGND1", type="string",default="Float",
                  help="CH1 input grounding. Float or Ground")
parser.add_option("--ig2",
                  dest="inputGND2", type="string",default="Float",
                  help="CH2 input grounding. Float or Ground")
parser.add_option("--id1",
                  dest="inputDiff1", type="string",default="SE",
                  help="CH1 input Differential A-B, or single ended A")
parser.add_option("--id2",
                  dest="inputDiff2", type="string",default="SE",
                  help="CH1 input Differential A-B, or single ended A")
parser.add_option("-w","--window",
                  dest="windowFunc", type="string",default="Hanning",
                  help="Window function: Uniform, Flattop, Hanning, BMH, Kaiser, Force/Exponential, User, [-T/2,T/2],[0,T/2] or [-T/4,T/4]")
parser.add_option("--xlin",
                  dest="xlog", default=True,
                  action="store_false",
                  help="Plot with linear x axis")
parser.add_option("--ylin",
                  dest="ylog", default=None,
                  action="store_false",
                  help="Plot with linear y axis")
parser.add_option("--xlog",
                  dest="xlog", default=True,
                  action="store_true",
                  help="Plot with logarithmic x axis")
parser.add_option("--ylog",
                  dest="ylog", default=None,
                  action="store_true",
                  help="Plot with logarithmic y axis")
parser.add_option("--title",
                  dest="title", type="string",default="",
                  help="Title of the measurement. The given string will be written into the parameter file.")
parser.add_option("--memo",
                  dest="memo", type="string",default="",
                  help="Use this option to note miscellaneous things.")



(options, args) = parser.parse_args()

#Convert number of points into available resolution
if options.numOfPoints <= 100:
    fRes=0 # Resolution is 100 points
elif options.numOfPoints <= 200:
    fRes=1 # Resolution is 200 points
elif options.numOfPoints <= 400:
    fRes=2 # Resolution is 400 points
else:
    fRes=3 # Resolution is 800 points


# Create a netGPIB class object
print('Connecting to '+str(options.ipAddress)+' ...'),
gpibObj=netgpib.netGPIB(options.ipAddress, options.gpibAddress, '\004',0)
print('done.')

#File names
dataFileName=options.filename+'.txt'

print('Measurement data will be written into '+dataFileName)
print('Setting up parameters for the measurement...') 


#Prepare for the FFT measurement

#Set output to GPIB
gpibObj.command("OUTX0")
time.sleep(0.1)

# Print IDN
print "Instrument ID: ",
idnString=gpibObj.query("*IDN?")
print idnString[:-1]
time.sleep(0.1)

#Set measurement parameters
if options.dual:
    gpibObj.command('DFMT1') # Dual display
    time.sleep(0.1)
    numDisp=2
else:
    gpibObj.command('DFMT0') # Dual display
    time.sleep(0.1)
    numDisp=1

#Input setup
if options.inputCoupling1 == "AC":
    icp1="1"
else:
    icp1="0"
gpibObj.command('I1CP'+icp1) #CH1 Input Coupling
time.sleep(0.1)

if options.inputCoupling2 == "AC":
    icp2="1"
else:
    icp2="0"
gpibObj.command('I2CP'+icp2) #CH2 Input Coupling
time.sleep(0.1)

if options.inputGND1 == "Float":
    igd1="0"
else:
    igd1="1"
gpibObj.command('I1GD'+igd1) #CH1 Input GND
time.sleep(0.1)

if options.inputGND2 == "Float":
    igd2="0"
else:
    igd2="1"
gpibObj.command('I2GD'+igd2) #CH2 Input GND
time.sleep(0.1)

if options.inputDiff1 == "SE":
    idf1="0"
else:
    idf1="1"
gpibObj.command('I1MD'+idf1) #CH1 Input A-B = 1; A = 0
time.sleep(0.1)

if options.inputDiff2 == "SE":
    idf1="0"
else:
    idf1="1"
gpibObj.command('I2MD'+idf1) #CH2 Input A-B = 1; A = 0
time.sleep(0.1)

gpibObj.command('A1RG0') #AutoRange Off
time.sleep(0.1)
gpibObj.command('A2RG0') #AutoRange Off
time.sleep(0.1)
gpibObj.command('I1AR0') #AutoRange Up Only
time.sleep(0.1)
gpibObj.command('I2AR0') #AutoRange Up Only
time.sleep(0.1)
gpibObj.command('A1RG1') #AutoRange On
time.sleep(0.1)
gpibObj.command('A2RG1') #AutoRange On
time.sleep(0.1)

gpibObj.command('I1AF1') #Anti-Aliasing filter On
time.sleep(0.1)
gpibObj.command('I2AF1') #Anti-Aliasing filter On
time.sleep(0.1)

#Common setup for both displays
gpibObj.command('MGRP2,0') # Measurement Group = FFT
gpibObj.command('ISRC1') # Input = Analog


#Set up each display
for disp in range(numDisp):
    
    gpibObj.command('ACTD'+str(disp)) # Change active display 
    time.sleep(0.1)
    gpibObj.command('MEAS'+str(disp)+','+str(disp)) # 0:FFT1, 1:FFT2    
    time.sleep(0.1)
    gpibObj.command('VIEW'+str(disp)+',0') #Log Magnitude
    time.sleep(0.1)
    gpibObj.command('UNDB'+str(disp)+',0') # dB OFF
    time.sleep(0.1)
    gpibObj.command('UNPK'+str(disp)+',2') # Vrms
    time.sleep(0.1)
    gpibObj.command('PSDU'+str(disp)+',1') # PSD ON
    time.sleep(0.1)
    gpibObj.command('DISP'+str(disp)+',1') # Live display on        
    
#Common setup
#gpibObj.command('FSPN2,'+options.bandWidth) # Frequency span
#time.sleep(0.1)
gpibObj.command('FLIN2,'+str(fRes)) #Frequency resolution
time.sleep(0.1)
gpibObj.command('FAVG2,1') #Averaging On
time.sleep(0.1)

avgModDict = {"None":0, "Vector":1, "RMS":2, "PeakHold":3}
if options.avgMode in avgModDict:
    avgModID=avgModDict[options.avgMode]
else:
    avgModID=2
gpibObj.command('FAVM2,'+str(avgModID)) # Averaging mode
time.sleep(0.1)

gpibObj.command('FAVT2,0') #Averaging Type = Linear
time.sleep(0.1)
gpibObj.command('FREJ2,1') #Overload Reject On
time.sleep(0.1)
gpibObj.command('FAVN2,'+str(options.numAvg)) # Number of Averaging
time.sleep(0.1)

winFuncDict = {"Uniform":0, "Flattop":1, "Hanning":2, "BMH":3, "Kaiser":4, "Force/Exponential":5, "User":6, "[-T/2,T/2]":7,"[0,T/2]":8,"[-T/4,T/4]":9}
if options.windowFunc in winFuncDict:
    winFuncID=winFuncDict[options.windowFunc]
else:
    winFuncID=2
gpibObj.command('FWIN2,'+str(winFuncID)) # Window function

# gpibObj.command('MEAS'+str(disp)+',3') # Measure 'Power Spectrum 2'

time.sleep(1)


#Loop to do all four measurements.
bandwidths = ['102.4kHz', '12.8kHz', '1.6kHz', '400Hz']
data = [ [], [] ]
freq = []
for b in bandwidths:

    #Set frequency for this measurement.
    gpibObj.command('FSPN2,'+str(b)) # Frequency span
    time.sleep(0.1)
    gpibObj.command('FBAS2,1') # Frequency base
    time.sleep(0.1)
    if (b == '400Hz') and (options.numAvg > 20):
        gpibObj.command('FAVN2,'+str(20)) # At low freq reduce # of averages
        time.sleep(0.1)
    
    #Start measurement
    print 'Measurement started'
    sys.stdout.flush()
    gpibObj.command('STRT') #Start measurement
    time.sleep(0.1)
    if (b == '102.4kHz'):
        time.sleep(0.3)
        gpibObj.command('A1RG0') #AutoRange Off
        time.sleep(0.1)
        gpibObj.command('A2RG0') #AutoRange Off
        time.sleep(0.3)
        gpibObj.command('A1RG1') #AutoRange On
        time.sleep(0.1)
        gpibObj.command('A2RG1') #AutoRange On
        time.sleep(0.3)
        
    #Wait for the measurement to end
    measuring = True
    avg=0
    print 'Averaging completed:',
    avgStatus=termstatus.statusTxt("0")
    while measuring:
        #Get status 
        measuring = not int(gpibObj.query('DSPS?1'))
        avg=int(gpibObj.query("NAVG?0"))
        avgStatus.update(str(avg))
        
        #if options.numAvg <= avg:
        #    measuring = False
        time.sleep(0.3)

    a=int(gpibObj.query("NAVG?0"))
    avgStatus.end(str(a))
    print('done')

    gpibObj.command('ASCL0') #Auto scale
    gpibObj.command('ASCL1') #Auto scale

    #Download Data
    time.sleep(0.5)

    for disp in range(numDisp):
        print('Downloading data from display #'+str(disp))
        (f,d)=SR785.downloadData(gpibObj, disp) 
        #Cut some of the garbage data off the low end
        del d[0:100]
        del f[0:100]
        data[disp].extend(d)
    
    freq.extend(f)

# Data indices
if ('+1.0000000e+001' in freq):
    FI1=freq.index('+1.0000000e+001')
if ('+1.0000000e+002' in freq):
    FI2=freq.index('+1.0000000e+002')
if ('+1.0000000e+003' in freq):
    FI3=freq.index('+1.0000000e+003')
if ('+1.0000000e+004' in freq):
    FI4=freq.index('+1.0000000e+004')
if ('+1.0000000e+005' in freq):
    FI5=freq.index('+1.0000000e+005')
    
# Print a summary table to fill in the test document
print '----- Noise Test Summary -----'
print 'Freq(Hz)\tMag(V/rtHz)'
if ('+1.0000000e+001' in freq):
    print str(float(freq[FI1]))+'\t\t'+str(float(data[0][FI1]))
if ('+1.0000000e+002' in freq):
    print str(float(freq[FI2]))+'\t\t'+str(float(data[0][FI2]))
if ('+1.0000000e+003' in freq):
    print str(float(freq[FI3]))+'\t\t'+str(float(data[0][FI3]))
if ('+1.0000000e+004' in freq):
    print str(float(freq[FI4]))+'\t\t'+str(float(data[0][FI4]))
if ('+1.0000000e+005' in freq):
    print str(float(freq[FI5]))+'\t'+str(float(data[0][FI5]))
print '----------'


# Open file
dataFile = open(dataFileName,'w')

print('Writing measurement data to file...')

#Deal with an empty title
if options.title == "":
    options.title = options.filename

#Write parameters to file
dataFile.write('# SR785 SUS Noise Measurement\n')
dataFile.write('Title: '+options.title+'\n')
dataFile.write('Memo: '+options.memo+'\n')
dataFile.write('Instrument ID: '+idnString+'\n')
dataFile.write('#---------- Measurement Setup ----------\n')
fSpan=gpibObj.query("FSPN?0")
fSpan=fSpan[:-1]
dataFile.write('Frequency Span: '+fSpan+'\n')

fRes=int(gpibObj.query("FLIN?0"))
fRes={0:100, 1:200, 2:400, 3:800}[fRes]
dataFile.write('Frequency Resolution: '+str(fRes)+'\n')

nAvg=int(gpibObj.query("NAVG?0"))
dataFile.write('Number of Averages: '+str(nAvg)+'\n')

avgMode={0:"None", 1:"Vector", 2:"RMS", 3:"PeakHold"}[int(gpibObj.query("FAVM?0"))]
dataFile.write('Averaging Mode: '+avgMode+'\n')

winFunc={0:"Uniform", 1:"Flattop", 2:"Hanning", 3:"BMH", 4:"Kaiser", 5:"Force/Exponential", 6:"User", 7:"[-T/2,T/2]",8:"[0,T/2]",9:"[-T/4,T/4]"}[int(gpibObj.query("FWIN?0"))]
dataFile.write('Window function: '+winFunc+'\n')

dataFile.write('\n')

SR785.getparam(gpibObj, options.filename, dataFile)

dataFile.write('#---------- Instrument Data -----------\n')
dataFile.write('# Column format = [Freq  Mag(V/rtHz)]\n')

#Write data to file
for i in range(len(freq)):
    dataFile.write(freq[i])
    dataFile.write(' '+data[0][i])
    dataFile.write('\n')

dataFile.close()

#Plot results for extra fanciness.
#print 'Plotting data...'
#fig = plt.figure()
#fig.suptitle('FFT Results')
#freqplot = fig.add_subplot(1,1,1)
#freqplot.loglog(freq,data[0])
#freqplot.set_xlabel('Frequency (Hz)')
#freqplot.set_ylabel('Magnitude (nVrms/Hz^1/2)')
#plt.show()

print "done."
gpibObj.close()

