#!/usr/bin/env python
#
# Transfer function script for suspensions electronics testing (4395A).  Modified from
# code by Yoichi Aso.
# cmreed 20100329
#
# 
# TFAG4395A.py [-f filename] [-i ip_address] [-a gpib_address] [{-s|--startfreq} start_freq]
#                 [{-e|--stopfreq} stop_freq] [-n|--numpoints num_of_points] [--sweep Linear|Log] 
#                 [{-x|--excamp} excitation_amplitude] [{-c|--settlecycle} settle_cycle]
#                 [{-t|--intcycle} integration_cycle] [--ic1 AC|DC ] [--ic2 AC|DC ]
#                 [--ig1 GND|Float ] [--ig2 GND|Float ] [--armode UpOnly|Tracking ] [--title Title]
#                 [--memo Memo]
#                 

import re
import sys
import math
import optparse
import time
import pdb
import netgpib
import AG4395A
import termstatus
#from pylab import *
#import matplotlib.pyplot as plt


#Parse options
usage = """usage: %prog [options]

This program executes a transfer function measurement using SR785.
Various measurement conditions can be set using options.
The measurement results will be saved in FILENAME.txt.
"""

filetime = time.strftime("%Y-%m-%d.%H%M%S",time.gmtime()) # Date string for data file (default)


# Options below set for SUS transfer function measurements
parser = optparse.OptionParser(usage=usage)
parser.add_option("-f", "--file", dest="filename",
                  help="Output file name without an extension", default = filetime)
parser.add_option("-i", "--ip",
                  dest="ipAddress", default="169.254.0.1",
                  help="IP address/Host name")
parser.add_option("-a", "--address",
                  dest="gpibAddress", type="int", default=10,
                  help="GPIB device address")
parser.add_option("-p", "--points",
                   dest="numPoints", type="str",default="801",
                  help="Number of points")
parser.add_option("-x", "--excamp",
                   dest="excAmp", type="str",default="-20dBm",
                  help="Excitation Amplitude")
parser.add_option("--AR", "--channelAR",
                  dest="chanAR", default=True,
                  action="store_true",
                  help="Which channels are you looking at (A/R, B/R, A/B)? Specify with '-AR'")
parser.add_option("--BR", "--channelBR",
                  dest="chanBR",default=False,
                  action="store_true",
                  help="Which channels are you looking at (A/R, B/R, A/B)? Specify with '-BR'")
parser.add_option("-v", "--averaging",
                   dest="numAvg", type="str",default="20",
                  help="Number of averages")
parser.add_option("--st", "--start",
                  dest="startF", type="str",default="100Hz",
                  help="Start frequency (in Hz, kHz, MHz, or dBm)")
parser.add_option("--fin", "--end",
                  dest="endF", type="str",default="10MHz",
                  help="End frequency (in Hz, kHz, MHz, or dBm)")
parser.add_option("--plot",
                  dest="plotData", default=False,
                  action="store_true",
                  help="Plot the downloaded data.")
parser.add_option("--lin",
                  dest="lin_p", default=None,
                  action="store_true",
                  help="Plot with linear axis")
parser.add_option("--semix",
                  dest="semix", default=None,
                  action="store_true",
                  help="Plot with logarithmic x axis")
parser.add_option("--semiy",
                  dest="semiy_p", default=None,
                  action="store_true",
                  help="Plot with logarithmic y axis")
parser.add_option("--loglog",
                  dest="loglog_p", default=True,
                  action="store_true",
                  help="Plot with both logarithmic axis")
parser.add_option("--magcorrection",
                  dest="magcorrection", type="float",default="0.0",
                  help="Correction to apply to data in dB (formula: DATA - CORRECTION, default: 0.0)")
parser.add_option("--phasecorrection",
                  dest="phasecorrection", type="float",default="0.0",
                  help="Correction to apply to phase in deg (formula: DATA - CORRECTION, default: 0.0)")
parser.add_option("--title",
                  dest="title", type="string",default="",
                  help="Title of the measurement. The given string will be written into the parameter file.")
parser.add_option("--memo",
                  dest="memo", type="string",default="",
                  help="Use this option to note miscellaneous things.")

(options, args) = parser.parse_args()
# Create a netGPIB class object
print('Connecting to '+str(options.ipAddress)+' ...'),
gpibObj=netgpib.netGPIB(options.ipAddress, options.gpibAddress, '\004',0)
print('done.')

#File names
dataFileName=options.filename+'.txt'

print('Measurement data will be written into '+dataFileName)
print('Setting up parameters for the measurement...') 

# Print IDN
print "Instrument ID: ",
idnString=gpibObj.query("*IDN?")
print idnString[:-1]
time.sleep(0.1)

#Prepare for the TF measurement

#Restore our settings to a known state.
gpibObj.command("PRES")
time.sleep(0.1)
#Set to Network Analyzer mode.
gpibObj.command("NA")
time.sleep(0.1)
#Use dual channel mode to get magnitude and phase at same time.
gpibObj.command("DUAC ON")
time.sleep(0.1)
#Set up the top display with magnitude first.
gpibObj.command("CHAN1")
time.sleep(0.1)
#Set correct channels to record.
if options.chanAR:
    MS = "AR"
if options.chanBR:
    MS = "BR"
gpibObj.command("MEAS "+str(MS))
#Set the number of averages.
gpibObj.command("AVERFACT "+str(options.numAvg))
time.sleep(0.1)
gpibObj.command("AVER ON")
#Start freq.
gpibObj.command("STAR "+str(options.startF))
time.sleep(0.1)
#Stop freq.
gpibObj.command("STOP "+str(options.endF))
time.sleep(0.1)
#Set to show logarithmic magnitude.
gpibObj.command("FMT LOGM")
time.sleep(0.1)
#Logarithmic frequency sweep.
gpibObj.command("SWPT LOGF")
time.sleep(0.1)
#Now set the bottom display to show phase.
gpibObj.command("CHAN2")
time.sleep(0.1)
gpibObj.command("MEAS "+str(MS))
#Set the number of averages.
gpibObj.command("AVERFACT "+str(options.numAvg))
time.sleep(0.1)
gpibObj.command("AVER ON")
#Start freq.
gpibObj.command("STAR "+str(options.startF))
time.sleep(0.1)
#Stop freq.
gpibObj.command("STOP "+str(options.endF))
time.sleep(0.1)
#Set excitation amplitude.
gpibObj.command("POWE "+str(options.excAmp))
time.sleep(0.1)
#Set to show phase, which is in deg by default.
gpibObj.command("FMT PHAS")
#Logarithmic frequency sweep.
gpibObj.command("SWPT LOGF")
time.sleep(0.1)
#Set number of points.
gpibObj.command("POIN "+str(options.numPoints))
time.sleep(0.1)
print 'Connect both cable ends (input and output) to a barrel to calibrate for cable delay.'
raw_input("Press a key when ready for calibration measurement.")
gpibObj.command("NUMG "+str(10))
print 'Calibrating...',

#Reconnect to GPIB. This is necessary because GPIB object tends
#to die after a sweep is started.
#print('Reconnecting to '+str(options.ipAddress)+' ...'),
gpibObj=netgpib.netGPIB(options.ipAddress, options.gpibAddress, '\004',0)
#print('done.')

measuring = True
while measuring:
    #Get status
    measuring = not int(gpibObj.query('HOLD?'))
    time.sleep(2)
    print('.'),
print '...calibration done.'
gpibObj.command("MKR ON")
time.sleep(0.1)
#Move to highest frequency point.
gpibObj.command("MKRP "+str(options.numPoints))
time.sleep(0.1)
#Adjust delay to zero out current measurement.
gpibObj.command("MKRDELA")
time.sleep(0.1)
raw_input("Press a key when reconnected to TF target's input & output.")

#Start measurement by running enough sweeps to cover our # of averages.
gpibObj.command("NUMG "+str(options.numAvg))
time.sleep(8)

#Wait for the measurement to end
measuring = True
print 'Running...'

#Reconnect to GPIB. This is necessary because GPIB object tends
#to die after a sweep is started.
#print('Reconnecting to '+str(options.ipAddress)+' ...'),
gpibObj=netgpib.netGPIB(options.ipAddress, options.gpibAddress, '\004',0)
#print('done.')

while measuring:
    #Get status
    measuring = not int(gpibObj.query('HOLD?'))
    time.sleep(2)
    print('.'),
print '...measurement done.'

#Get list of frequency points
receivedData = gpibObj.query('OUTPSWPRM?',1024)
freq=re.findall(r'[-+.E0-9]+', receivedData, re.M)

data=[]
mag=[]
phase=[]
print 'Retrieving data...',
#Doing it with the marker allows us to get it in mag/phase format.
gpibObj.command('MKR ON')
time.sleep(0.1)
gpibObj.command('CHAN1')
for f in freq:
    gpibObj.command('MKRPRM '+str(f)+'Hz')
    mag.append(gpibObj.query('MKRVAL?'))
data.append(mag)
gpibObj.command('CHAN2')
for f in freq:
    gpibObj.command('MKRPRM '+str(f)+'Hz')
    phase.append(gpibObj.query('MKRVAL?'))
data.append(phase)
magCorrString=str(options.magcorrection)

# Apply correction factor to magnitude data
# Convert dB data to float
floatdata = [float(item) for item in data[0]]
# Correct dB data using options
caldata = map((lambda x: x-options.magcorrection), floatdata)
# Cast float data back to string
strcaldata = map((lambda x: "%e" % x), caldata)

# Apply correction factor to phase data
# Convert dB data to float
floatphasedata = [float(item) for item in data[1]]
# Correct dB data using options
calphasedata = map((lambda x: x-options.phasecorrection), floatphasedata)
# Cast float data back to string
strcalphasedata = map((lambda x: "%e" % x), calphasedata)

phaseCorrString=str(options.phasecorrection)

# Find data indexes for nearest values to set points, if they're in range.
freqindexlist = [10,100,1000,10000,100000,300000,1000000,10000000,100000000]
FI=[]
for f in freqindexlist:
    if ((f>=float(freq[0])) and (f<=float(freq[len(freq)-1]))):
        i = 0
        while (float(freq[i])<f):
            i+=1
        FI.append(i)
        
# Print a summary table to fill in the test document
print '----- Transfer Function Test Summary -----'
print 'Calibrated Mag(dB) = Mag(dB) - '+magCorrString+'(dB)'
print
print 'Freq(Hz)\tMag(dB)\t\tCalibratedMag(dB)'
for i in FI:
    print str(float(freq[i]))+'\t\t'+str(float(data[0][i]))+'\t'+str(caldata[i])

print
print 'Calibrated Phase(deg) = Phase(deg) - '+phaseCorrString+'(deg)'
print
print 'Freq(Hz)\tPhase(deg)\tCalibratedPhase(deg)'


for i in FI:
    print str(float(freq[i]))+'\t\t'+str(float(data[1][i]))+'\t'+str(calphasedata[i])
print '----------'

#Open file
dataFile = open(dataFileName,'w')

#Write to the data file
print('Writing measurement data to file...')

#Deal with an empty title
if options.title == "":
    options.title = options.filename

#Write parameters to file
dataFile.write('# SR785 SUS Transfer Function Measurement\n')
dataFile.write('Title: '+options.title+'\n')
dataFile.write('Memo: '+options.memo+'\n')
dataFile.write('Instrument ID: '+idnString+'\n')
dataFile.write('#---------- Measurement Setup ------------\n')
dataFile.write('Start frequency = '+options.startF+'\n')
dataFile.write('Stop frequency = '+options.endF+'\n')
dataFile.write('Number of frequency points = '+str(options.numPoints)+'\n')
dataFile.write('Number of averages = '+str(options.numAvg)+'\n')
dataFile.write('Excitation amplitude = '+options.excAmp+'\n')
dataFile.write('\n')

AG4395A.getparam(gpibObj, options.filename, dataFile)

dataFile.write('#---------- Transfer Function Test Summary -----------\n')
dataFile.write('Calibrated Mag(dB) = Mag(dB) - '+magCorrString+'(dB)\n')
dataFile.write('Calibrated Phase(deg) = Phase(deg) - '+phaseCorrString+'(deg)\n')
dataFile.write('[Frequency (Hz) Mag(dB) CalibratedMag(db) Phase(deg) CalibratedPhase(deg)]\n')
for i in FI:
    dataFile.write(freq[i]+' '+data[0][i]+' '+strcaldata[i]+' '+data[1][i]+' '+strcalphasedata[i]+'\n')
dataFile.write('\n')

dataFile.write('#---------- Instrument Data -----------\n')
dataFile.write('# Column format = [Freq  Mag(dB) CalibratedMag(dB) Phase(deg) CalibratedPhase(deg)]\n')

#Write data to file
for i in range(len(freq)):
    dataFile.write(freq[i])
    dataFile.write(' '+data[0][i])
    dataFile.write(' '+strcaldata[i])
    dataFile.write(' '+data[1][i])
    dataFile.write(' '+strcalphasedata[i])
    dataFile.write('\n')

dataFile.close()

#Plot results for extra fanciness.
#print 'Plotting data...'
#fig = plt.figure()
#fig.suptitle('Transfer Function Results')
#freqplot = fig.add_subplot(2,1,1)
#freqplot.semilogx(freq,data[0])
#freqplot.set_xlabel('Frequency (Hz)')
#freqplot.set_ylabel('Magnitude (dB)')
#phaseplot = fig.add_subplot(2,1,2)
#phaseplot.semilogx(freq,data[1])
#phaseplot.set_xlabel('Frequency (Hz)')
#phaseplot.set_ylabel('Phase (deg)')
#plt.show()


print "Done."
gpibObj.close()

